/*
 * main.c
 *
 *      Author: Tomek
 */

#include <stm32l/rcc.h>
#include <stm32l/nvic.h>
#include <stm32l/gpio.h>
#include <stm32l/stm32l.h>
#include <stm32l/iwdg.h>
#include <stm32l/flash.h>

/* constant expressing got duration */
#define DOT_DURATION 						50000

/* bit that controls rf output */
static bitband_t rf = BITBAND_PERIPH(&GPIOA->MODER, 17);

/* encoded morse letters from A to Z */
const uint8_t morse_letters[] = {
		0b10000010, 0b01110100, 0b01010100, 0b01100011,
		0b10000001, 0b11010100, 0b00100011, 0b11110100,
		0b11000010, 0b10000100, 0b01000011, 0b10110100,
		0b00000010, 0b01000010, 0b00000011, 0b10010100,
		0b00100100, 0b10100011, 0b11100011, 0b00000001,
		0b00100011, 0b11100100, 0b10000011, 0b01100100,
		0b01000100, 0b00110100
};

/* timing is not guaranteed :) */
void simple_delay(uint32_t us)
{
	/* simple delay loop */
	while (us--) {
		asm volatile ("nop");
	}
}

/* sends morse letter */
void send_morse(char c)
{
	/* get code */
	uint8_t code = morse_letters[c - 'a'], i = code & 0x7;
	/* process every bit of morse code */
	while (i--) {
		/* enable rf */
		*(rf) = 1;
		/* dot */
		if (code & 0x80) {
			simple_delay(DOT_DURATION);
		/* dash */
		} else {
			simple_delay(DOT_DURATION * 3);
		}
		/* disable rf */
		*(rf) = 0;
		/* dot space */
		simple_delay(DOT_DURATION);
		/* next bit */
		code = code << 1;
	}
}

/* system entry point */
int main(void)
{
	/* gpio init struct */
	gpio_init_t gpio;

	/* reset rcc */
	RCC_DeInit();

	/* start hsi clock */
	RCC_HSECmd(ENABLE);
	/* wait till it stabilizes */
	while (!RCC_GetHSEStatus());

	/* configure pll to give out 96 MHz */
	RCC_PLLConfig(RCC_PLLSOURCE_HSE, RCC_PLLMUL_12, RCC_PLLDIV_2);
	/* start pll */
	RCC_PLLCmd(ENABLE);
	/* stabilize pll */
	while (!RCC_GetPLLStatus());

	/* enable clock to GPIOC */
	RCC_AHBPeriphClockCmd(AHB_GPIOA | AHB_GPIOC, ENABLE);

	/* initialize gpio structure */
	GPIO_StructInit(&gpio);
	/* use pin 13 */
	gpio.pins = GPIO_P8;
	/* mode: alternate setting */
	gpio.mode = GPIO_ALT;
	/* system */
	gpio.af = GPIO_AF_SYSTEM;
	/* output type: push-pull */
	gpio.speed = GPIO_OS_40M;
	/* apply configuration */
	GPIO_Init(GPIOA, &gpio);

	/* configure mco pin */
	RCC_MCOConfig(RCC_MCOSOURCE_PLLCLK, RCC_MCODIV_1);

	/* an example string */
	char string[] = "mightydevices", *ptr;

	/* main program loop */
	for (;;) {
		/* reset pointer */
		ptr = string;
		/* send letter by letter */
		while (*ptr) {
			send_morse(*(ptr++));
		}
		/* delay between words */
		simple_delay(DOT_DURATION * 20);
	}

	/* never reached */
	return 0;
}


